/*
    parse_aux.c - Part of psiconv, a PSION 5 file formats converter
    Copyright (c) 1999  Frodo Looijaard <frodol@dds.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "config.h"
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "parse.h"
#include "parse_routines.h"
#include "data.h"

void psiconv_fatal(int level, psiconv_u32 off, const char *format,...)
{
  va_list ap;
  va_start(ap,format);
  fprintf(stderr,"Fatal error (offset %08x): ",off);
  vfprintf(stderr,format,ap);
  fputs("\n",stderr);
  va_end(ap);
  exit(1);
}

void psiconv_progress(int level, psiconv_u32 off, const char *format,...)
{
  int i;
  va_list ap;
  va_start(ap,format);
  if (psiconv_verbosity >= PSICONV_VERB_PROGRESS) {
    fprintf(stderr,"%08x ",off);
    for (i = 0; i < level; i++)
      fputs("=",stderr);
    fprintf(stderr,"> ");
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}

void psiconv_warn(int level, psiconv_u32 off, const char *format,...)
{
  va_list ap;
  va_start(ap,format);
  if (psiconv_verbosity >= PSICONV_VERB_WARN) {
    fprintf(stderr,"WARNING (offset %08x): ",off);
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}

void psiconv_debug(int level, psiconv_u32 off, const char *format,...)
{
  int i;
  va_list ap;
  va_start(ap,format);
  if (psiconv_verbosity >= PSICONV_VERB_DEBUG) {
    fprintf(stderr,"%08x ",off);
    for (i = 0; i < level; i++)
      fputs("-",stderr);
    fprintf(stderr,"> ");
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}

char *psiconv_make_printable(const char *s)
{
  int i;
  char *res = malloc(strlen(s) + 1);
  for (i = 0; i < strlen(s); i ++)
    if (s[i] < 0x20 || s[i] >= 0x7f)
      res[i] = '.';
    else
      res[i] = s[i];
  res[strlen(s)] = 0;
  return res;
}


